"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NavigatorRouteView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _tsEventEmitter = require("@dominicstop/ts-event-emitter");

var _NavigatorRouteViewEventEmitter = require("../types/NavigatorRouteViewEventEmitter");

var _RouteComponentsWrapper = require("../wrapper_components/RouteComponentsWrapper");

var _NavigatorRouteContentWrapper = require("../wrapper_components/NavigatorRouteContentWrapper");

var _RouteHeaderWrapper = require("../wrapper_components/RouteHeaderWrapper");

var _RNINavigatorRouteView = require("../native_components/RNINavigatorRouteView");

var _RNINavigatorRouteViewModule = require("../native_modules/RNINavigatorRouteViewModule");

var Helpers = _interopRequireWildcard(require("../functions/Helpers"));

var _CompareRouteOptions = require("../functions/CompareRouteOptions");

var _CompareUtilities = require("../functions/CompareUtilities");

var _NavigatorError = require("../functions/NavigatorError");

var _NavigationContext = require("../context/NavigationContext");

var _NavigatorUIConstantsContext = require("../context/NavigatorUIConstantsContext");

var _LibraryConstants = require("../constants/LibraryConstants");

var _LibEnv = require("../constants/LibEnv");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

;
var RouteStatus;

(function (RouteStatus) {
  RouteStatus["INIT"] = "INIT";
  RouteStatus["ROUTE_PUSHING"] = "ROUTE_PUSHING";
  RouteStatus["ROUTE_PUSHED"] = "ROUTE_PUSHED";
  RouteStatus["ROUTE_POPPING"] = "ROUTE_POPPING";
  RouteStatus["ROUTE_POPPED"] = "ROUTE_POPPED";
  RouteStatus["ROUTE_BLURRING"] = "ROUTE_BLURRING";
  RouteStatus["ROUTE_BLURRED"] = "ROUTE_BLURRED";
  RouteStatus["ROUTE_FOCUSING"] = "ROUTE_FOCUSING";
  RouteStatus["ROUTE_FOCUSED"] = "ROUTE_FOCUSED";
  RouteStatus["UNMOUNTED"] = "UNMOUNTED";
})(RouteStatus || (RouteStatus = {}));

;

//#endregion
class NavigatorRouteView extends _react.default.Component {
  //#region - Property Declarations

  /** is incremented whenever `props.routeProps` changes */
  // references
  //#endregion
  constructor(_props) {
    super(_props);

    _defineProperty(this, "routeStatus", void 0);

    _defineProperty(this, "routePropsUpdateIndex", 0);

    _defineProperty(this, "isPushed", false);

    _defineProperty(this, "_emitter", void 0);

    _defineProperty(this, "_nativeRef", void 0);

    _defineProperty(this, "_navigatorRef", void 0);

    _defineProperty(this, "_routeContentRef", void 0);

    _defineProperty(this, "_routeViewPortalRef", void 0);

    _defineProperty(this, "_routeHeaderWrapperRef", void 0);

    _defineProperty(this, "_routeComponentsWrapperRef", void 0);

    _defineProperty(this, "getRefToNavigator", () => {
      return this.props.getRefToNavigator();
    });

    _defineProperty(this, "getRouteRef", () => {
      return this;
    });

    _defineProperty(this, "getEmitterRef", () => {
      return this._emitter;
    });

    _defineProperty(this, "getPortalRef", () => {
      return this._routeViewPortalRef;
    });

    _defineProperty(this, "getRouteOptions", () => {
      var _this$_routeViewPorta, _routeOptions$statusB, _routeOptions$routeCo, _ref, _routeOptions$automat, _ref2, _props$transitionConf, _ref3, _props$transitionConf2, _ref4, _routeOptions$routeTi, _routeOptions$prompt, _routeOptions$largeTi, _routeOptions$searchB, _routeOptions$navBarA, _routeOptions$navigat, _routeOptions$allowTo, _routeOptions$navBarB, _ref5, _routeOptions$navBarB2, _ref6, _routeOptions$navBarB3, _routeOptions$leftIte, _routeOptions$backBut, _routeOptions$backBut2, _routeOptions$hidesBa, _routeOptions$applyBa;

      const props = this.props;
      const portalProps = (_this$_routeViewPorta = this._routeViewPortalRef) === null || _this$_routeViewPorta === void 0 ? void 0 : _this$_routeViewPorta.props;
      const {
        routeOptionsDefault
      } = this.props;
      const {
        routeOptions
      } = this.state; // check if portal has custom nav bar items

      const hasNavBarLeftItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarLeftItem) != null;
      const hasNavBarRightItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarRightItem) != null;
      return {
        // ------------------------------------------------------
        // Technically, this whole thing could be done like this:
        // `{...routeOptionsDefault, ...routeOptions}`
        // but it's less clear/explicit, idk refactor this later.
        // ------------------------------------------------------
        statusBarStyle: (_routeOptions$statusB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.statusBarStyle) !== null && _routeOptions$statusB !== void 0 ? _routeOptions$statusB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.statusBarStyle,
        routeContainerStyle: (_routeOptions$routeCo = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.routeContainerStyle) !== null && _routeOptions$routeCo !== void 0 ? _routeOptions$routeCo : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeContainerStyle,
        automaticallyAddHorizontalSafeAreaInsets: (_ref = (_routeOptions$automat = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.automaticallyAddHorizontalSafeAreaInsets) !== null && _routeOptions$automat !== void 0 ? _routeOptions$automat : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.automaticallyAddHorizontalSafeAreaInsets) !== null && _ref !== void 0 ? _ref : true,
        // #region - Transition Config |
        // ----------------------------*
        transitionConfigPush: (_ref2 = (_props$transitionConf = props.transitionConfigPushOverride) !== null && _props$transitionConf !== void 0 ? _props$transitionConf : routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.transitionConfigPush) !== null && _ref2 !== void 0 ? _ref2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPush,
        transitionConfigPop: (_ref3 = (_props$transitionConf2 = props.transitionConfigPopOverride) !== null && _props$transitionConf2 !== void 0 ? _props$transitionConf2 : routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.transitionConfigPop) !== null && _ref3 !== void 0 ? _ref3 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPop,
        // #endregion -------------*
        // #region - Navbar Config |
        // ------------------------*
        routeTitle: (_ref4 = (_routeOptions$routeTi = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.routeTitle) !== null && _routeOptions$routeTi !== void 0 ? _routeOptions$routeTi : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeTitle) !== null && _ref4 !== void 0 ? _ref4 : props.routeKey,
        prompt: (_routeOptions$prompt = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.prompt) !== null && _routeOptions$prompt !== void 0 ? _routeOptions$prompt : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.prompt,
        largeTitleDisplayMode: (_routeOptions$largeTi = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.largeTitleDisplayMode) !== null && _routeOptions$largeTi !== void 0 ? _routeOptions$largeTi : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.largeTitleDisplayMode,
        searchBarConfig: (_routeOptions$searchB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.searchBarConfig) !== null && _routeOptions$searchB !== void 0 ? _routeOptions$searchB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.searchBarConfig,
        // #endregion ----------------------------------*
        // #region - `NavigationConfigOverride`-related |
        // ---------------------------------------------*
        navBarAppearanceOverride: (_routeOptions$navBarA = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarAppearanceOverride) !== null && _routeOptions$navBarA !== void 0 ? _routeOptions$navBarA : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarAppearanceOverride,
        navigationBarVisibility: (_routeOptions$navigat = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navigationBarVisibility) !== null && _routeOptions$navigat !== void 0 ? _routeOptions$navigat : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navigationBarVisibility,
        allowTouchEventsToPassThroughNavigationBar: (_routeOptions$allowTo = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.allowTouchEventsToPassThroughNavigationBar) !== null && _routeOptions$allowTo !== void 0 ? _routeOptions$allowTo : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.allowTouchEventsToPassThroughNavigationBar,
        // #endregion ------------------*
        // #region - Navbar Item Config |
        // -----------------------------*
        navBarButtonBackItemConfig: (_routeOptions$navBarB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonBackItemConfig) !== null && _routeOptions$navBarB !== void 0 ? _routeOptions$navBarB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonBackItemConfig,
        navBarButtonLeftItemsConfig: (_ref5 = (_routeOptions$navBarB2 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonLeftItemsConfig) !== null && _routeOptions$navBarB2 !== void 0 ? _routeOptions$navBarB2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonLeftItemsConfig) !== null && _ref5 !== void 0 ? _ref5 : // custom left bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarLeftItem ? [{
          type: 'CUSTOM'
        }] : undefined,
        navBarButtonRightItemsConfig: (_ref6 = (_routeOptions$navBarB3 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonRightItemsConfig) !== null && _routeOptions$navBarB3 !== void 0 ? _routeOptions$navBarB3 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonRightItemsConfig) !== null && _ref6 !== void 0 ? _ref6 : // custom right bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarRightItem ? [{
          type: 'CUSTOM'
        }] : undefined,
        // #endregion ------------------------------*
        // #region - Navbar back button item config |
        // -----------------------------------------*
        leftItemsSupplementBackButton: (_routeOptions$leftIte = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.leftItemsSupplementBackButton) !== null && _routeOptions$leftIte !== void 0 ? _routeOptions$leftIte : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.leftItemsSupplementBackButton,
        backButtonTitle: (_routeOptions$backBut = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.backButtonTitle) !== null && _routeOptions$backBut !== void 0 ? _routeOptions$backBut : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonTitle,
        backButtonDisplayMode: (_routeOptions$backBut2 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.backButtonDisplayMode) !== null && _routeOptions$backBut2 !== void 0 ? _routeOptions$backBut2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonDisplayMode,
        hidesBackButton: (_routeOptions$hidesBa = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.hidesBackButton) !== null && _routeOptions$hidesBa !== void 0 ? _routeOptions$hidesBa : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.hidesBackButton,
        applyBackButtonConfigToCurrentRoute: (_routeOptions$applyBa = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.applyBackButtonConfigToCurrentRoute) !== null && _routeOptions$applyBa !== void 0 ? _routeOptions$applyBa : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.applyBackButtonConfigToCurrentRoute // #endregion

      };
    });

    _defineProperty(this, "getRouteNavigationObject", () => {
      const props = this.props;
      const routeOptions = this.getRouteOptions();
      return {
        routeOptions,
        // pass down route details/data
        routeKey: props.routeKey,
        routeIndex: props.routeIndex,
        routeProps: props.routeProps,
        // pass down navigator commands
        push: this._navigatorRef.push,
        pop: this._navigatorRef.pop,
        popToRoot: this._navigatorRef.popToRoot,
        removeRoute: this._navigatorRef.removeRoute,
        removeRoutes: this._navigatorRef.removeRoutes,
        replaceRoute: this._navigatorRef.replaceRoute,
        insertRoute: this._navigatorRef.insertRoute,
        setRoutes: this._navigatorRef.setRoutes,
        setNavigationBarHidden: this._navigatorRef.setNavigationBarHidden,
        // pass down misc. navigator commands
        sendCustomCommandToNative: this._navigatorRef.sendCustomCommandToNative,
        getNavigatorConstants: this._navigatorRef.getNavigatorConstants,
        getActiveRoutes: this._navigatorRef.getActiveRoutes,
        dismissModal: this._navigatorRef.dismissModal,
        // pass down convenience navigator commands
        replacePreviousRoute: this._navigatorRef.replacePreviousRoute,
        replaceCurrentRoute: this._navigatorRef.replaceCurrentRoute,
        removePreviousRoute: this._navigatorRef.removePreviousRoute,
        removeAllPrevRoutes: this._navigatorRef.removeAllPrevRoutes,
        // navigator route commands
        getRouteOptions: this.getRouteOptions,
        setRouteOptions: this.setRouteOptions,
        setHidesBackButton: this.setHidesBackButton,
        getRouteConstants: this.getRouteConstants,
        // pass down 'get ref' functions
        getRefToRoute: this._handleGetRefToRoute,
        getRefToNavigator: props.getRefToNavigator,
        getRefToNavRouteEmitter: this._handleGetRefToNavRouteEmitter
      };
    });

    _defineProperty(this, "setRouteOptions", async (routeOptions) => {
      await Helpers.setStateAsync(this, prevState => ({ ...prevState,
        routeOptions,
        updateIndex: prevState.updateIndex + 1
      }));
    });

    _defineProperty(this, "setRouteViewPortalRef", ref => {
      var _this$_routeHeaderWra, _this$_routeComponent;

      this._routeViewPortalRef = ref;
      (_this$_routeHeaderWra = this._routeHeaderWrapperRef) === null || _this$_routeHeaderWra === void 0 ? void 0 : _this$_routeHeaderWra.setPortalRef(ref);
      (_this$_routeComponent = this._routeComponentsWrapperRef) === null || _this$_routeComponent === void 0 ? void 0 : _this$_routeComponent.setPortalRef(ref);
      this.setState({
        hasRoutePortal: true
      });
    });

    _defineProperty(this, "setHidesBackButton", async (isHidden, animated) => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.libraryError,
            message: "'NavigatorRouteView' is not mounted"
          });
        }

        ;

        if (this._nativeRef == null) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.libraryError,
            message: "'this._nativeRef' is not set"
          });
        }

        ;
        await Helpers.promiseWithTimeout(1000, _RNINavigatorRouteViewModule.RNINavigatorRouteViewModule.setHidesBackButton(Helpers.getNativeNodeHandle(this._nativeRef), isHidden, animated));
      } catch (e) {
        throw new _NavigatorError.NavigatorError(e);
      }

      ;
    });

    _defineProperty(this, "getRouteConstants", async () => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.libraryError,
            message: "'NavigatorRouteView' is not mounted"
          });
        }

        ;

        if (this._nativeRef == null) {
          throw new _NavigatorError.NavigatorError({
            code: _NavigatorError.NavigatorErrorCodes.libraryError,
            message: "'this._nativeRef' is not set"
          });
        }

        ;
        const result = await Helpers.promiseWithTimeout(1000, _RNINavigatorRouteViewModule.RNINavigatorRouteViewModule.getRouteConstants(Helpers.getNativeNodeHandle(this._nativeRef)));
        return result;
      } catch (e) {
        throw new _NavigatorError.NavigatorError(e);
      }

      ;
    });

    _defineProperty(this, "_handleGetRefToRoute", () => {
      return this;
    });

    _defineProperty(this, "_handleGetRefToNavRouteEmitter", () => {
      return this._emitter;
    });

    _defineProperty(this, "_handleOnRouteWillPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPING;
    });

    _defineProperty(this, "_handleOnRouteDidPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPED;
    });

    _defineProperty(this, "_handleOnRouteWillPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHING;
    });

    _defineProperty(this, "_handleOnRouteDidPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHED;
    });

    _defineProperty(this, "_handleOnPressNavBarLeftItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarLeftItem, event);
    });

    _defineProperty(this, "_handleOnPressNavBarRightItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onPressNavBarRightItem, event);
    });

    _defineProperty(this, "_handleOnRouteWillFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSING;
    });

    _defineProperty(this, "_handleOnRouteDidFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSED;
      this.isPushed = true;
    });

    _defineProperty(this, "_handleOnRouteWillBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteWillBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRING;
    });

    _defineProperty(this, "_handleOnRouteDidBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onRouteDidBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRED;
    });

    _defineProperty(this, "_handleOnUpdateSearchResults", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onUpdateSearchResults, event);
    });

    _defineProperty(this, "_handleOnSearchBarCancelButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onSearchBarCancelButtonClicked, event);
    });

    _defineProperty(this, "_handleOnSearchBarSearchButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(_NavigatorRouteViewEventEmitter.NavigatorRouteViewEvents.onSearchBarSearchButtonClicked, event);
    });

    _defineProperty(this, "_renderRouteContents", navigation => {
      var _this$_routeViewPorta2;

      const props = this.props;
      const routeOptions = navigation.routeOptions;
      const routeContent = props.renderRouteContent();

      const routeContentWithProps = /*#__PURE__*/_react.default.cloneElement(routeContent, {
        navigation,
        // store a ref to this element
        ...(Helpers.isClassComponent(routeContent) && {
          ref: node => {
            this._routeContentRef = node;
          }
        })
      });

      const portalProps = (_this$_routeViewPorta2 = this._routeViewPortalRef) === null || _this$_routeViewPorta2 === void 0 ? void 0 : _this$_routeViewPorta2.props;
      const hasRouteHeader = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderRouteHeader) != null || props.renderRouteHeader != null;
      return /*#__PURE__*/_react.default.createElement(_NavigatorUIConstantsContext.NavigatorUIConstantsContext.Consumer, null, context => /*#__PURE__*/_react.default.createElement(_NavigatorRouteContentWrapper.NavigatorRouteContentWrapper, {
        isInFocus: props.isInFocus,
        routePropsUpdateIndex: this.routePropsUpdateIndex,
        routeContainerStyle: routeOptions.routeContainerStyle,
        automaticallyAddHorizontalSafeAreaInsets: routeOptions.automaticallyAddHorizontalSafeAreaInsets,
        safeAreaInsets: context.safeAreaInsets,
        hasRouteHeader: hasRouteHeader
      }, routeContentWithProps, /*#__PURE__*/_react.default.createElement(_RouteHeaderWrapper.RouteHeaderWrapper, {
        ref: r => {
          this._routeHeaderWrapperRef = r;
        },
        navigation: navigation,
        getPortalRef: this.getPortalRef
      })));
    });

    this.routeStatus = RouteStatus.INIT;
    this._emitter = new _tsEventEmitter.TSEventEmitter();
    this._navigatorRef = _props.getRefToNavigator();
    this.state = {
      updateIndex: 0,
      hasRoutePortal: false,
      routeOptions: {}
    };
  }

  shouldComponentUpdate(nextProps, nextState) {
    const prevProps = this.props;
    const prevState = this.state;
    const didRoutePropsChange = !_CompareUtilities.CompareUtilities.unwrapAndShallowCompareObject(prevProps.routeProps, nextProps.routeProps);
    if (didRoutePropsChange) this.routePropsUpdateIndex++; // note: short-circuit eval so:
    // * value types first
    // * complex/nested objects last

    return didRoutePropsChange // props: compare `routeIndex` - routes were re-arranged/sorted
    || prevProps.routeIndex !== nextProps.routeIndex // props: compare focus change
    || prevProps.isInFocus !== nextProps.isInFocus // state: compare `hasRoutePortal` - portal comp. added/removed 
    || prevState.hasRoutePortal !== nextState.hasRoutePortal // state: compare `updateIndex` - force an update
    || prevState.updateIndex !== nextState.updateIndex // props: compare "push transition config"
    || !_CompareRouteOptions.CompareRouteTransitionPushConfig.unwrapAndCompare(prevProps.transitionConfigPushOverride, nextProps.transitionConfigPushOverride) // props: compare "pop transition config"
    || !_CompareRouteOptions.CompareRouteTransitionPopConfig.unwrapAndCompare(prevProps.transitionConfigPopOverride, nextProps.transitionConfigPopOverride) // props: compare "default route options"
    || !_CompareRouteOptions.CompareRouteOptions.unwrapAndCompare(prevProps.routeOptionsDefault, nextProps.routeOptionsDefault);
  }

  componentWillUnmount() {
    //#region - 🐞 DEBUG 🐛
    _LibEnv.LIB_ENV.debugLog && console.log("LOG/JS - NavigatorRouteView, componentWillUnmount" + " - routeID: ".concat(this.props.routeID) + " - routeIndex: ".concat(this.props.routeIndex) + " - routeKey: ".concat(this.props.routeKey)); //#endregion

    this.routeStatus = RouteStatus.UNMOUNTED;
  }

  updateRouteComponents() {
    var _this$_routeComponent2, _this$_routeHeaderWra2;

    // don't allow update until it's pushed
    if (!this.isPushed) return;
    (_this$_routeComponent2 = this._routeComponentsWrapperRef) === null || _this$_routeComponent2 === void 0 ? void 0 : _this$_routeComponent2.requestUpdate();
    (_this$_routeHeaderWra2 = this._routeHeaderWrapperRef) === null || _this$_routeHeaderWra2 === void 0 ? void 0 : _this$_routeHeaderWra2.requestUpdate();
  }

  render() {
    var _routeOptions$allowTo2;

    const props = this.props;
    const navigation = this.getRouteNavigationObject();
    const routeOptions = navigation.routeOptions; //#region - 🐞 DEBUG 🐛

    _LibEnv.LIB_ENV.debugLogRender && console.log("LOG/JS - NavigatorRouteView, render" + " - routeID: ".concat(props.routeID) + " - routeKey: ".concat(props.routeKey) + " - routeKey: ".concat(props.routeIndex)); //#endregion

    return /*#__PURE__*/_react.default.createElement(_NavigationContext.NavigationContext.Provider, {
      value: {
        navigation,
        navigatorID: props.navigatorID,
        routeID: props.routeID
      }
    }, /*#__PURE__*/_react.default.createElement(_RNINavigatorRouteView.RNINavigatorRouteView, _extends({
      style: styles.navigatorRouteView,
      ref: r => {
        this._nativeRef = r;
      },
      routeID: props.routeID,
      nativeID: _LibraryConstants.NativeIDKeys.NavRouteItem,
      routeKey: props.routeKey,
      routeIndex: props.routeIndex // Route Native Events: Push/Pop Events
      ,
      onRouteWillPop: this._handleOnRouteWillPop,
      onRouteDidPop: this._handleOnRouteDidPop,
      onRouteWillPush: this._handleOnRouteWillPush,
      onRouteDidPush: this._handleOnRouteDidPush // Route Native Events: Focus/Blur
      ,
      onRouteWillFocus: this._handleOnRouteWillFocus,
      onRouteDidFocus: this._handleOnRouteDidFocus,
      onRouteWillBlur: this._handleOnRouteWillBlur,
      onRouteDidBlur: this._handleOnRouteDidBlur // Route Native Events: Navbar Item `onPress`
      ,
      onPressNavBarLeftItem: this._handleOnPressNavBarLeftItem,
      onPressNavBarRightItem: this._handleOnPressNavBarRightItem // Route Native Events: Search
      ,
      onUpdateSearchResults: this._handleOnUpdateSearchResults,
      onSearchBarCancelButtonClicked: this._handleOnSearchBarCancelButtonClicked,
      onSearchBarSearchButtonClicked: this._handleOnSearchBarSearchButtonClicked // pass down navbar item config, back button item config, etc.

    }, routeOptions, {
      allowTouchEventsToPassThroughNavigationBar: (_routeOptions$allowTo2 = routeOptions.allowTouchEventsToPassThroughNavigationBar) !== null && _routeOptions$allowTo2 !== void 0 ? _routeOptions$allowTo2 : false
    }), this._renderRouteContents(navigation), /*#__PURE__*/_react.default.createElement(_RouteComponentsWrapper.RouteComponentsWrapper, {
      ref: r => {
        this._routeComponentsWrapperRef = r;
      },
      navigation: navigation,
      getPortalRef: this.getPortalRef // render nav bar items
      ,
      renderNavBarLeftItem: props.renderNavBarLeftItem,
      renderNavBarRightItem: props.renderNavBarRightItem,
      renderNavBarTitleItem: props.renderNavBarTitleItem
    })));
  }

}

exports.NavigatorRouteView = NavigatorRouteView;
;

class RouteViewUtils {
  static isRouteReady(routeStatus) {
    return routeStatus !== RouteStatus.INIT && routeStatus !== RouteStatus.UNMOUNTED;
  }

  static isRouteFocusingOrFocused(routeStatus) {
    return routeStatus === RouteStatus.ROUTE_PUSHING || routeStatus === RouteStatus.ROUTE_PUSHED || routeStatus === RouteStatus.ROUTE_FOCUSING || routeStatus === RouteStatus.ROUTE_FOCUSED;
  }

}

;

const styles = _reactNative.StyleSheet.create({
  navigatorRouteView: { ..._reactNative.StyleSheet.absoluteFillObject,
    // don't show
    opacity: 0
  }
});
//# sourceMappingURL=NavigatorRouteView.js.map